<?php

namespace Drupal\address_autocomplete_france\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Http\Adapter\Guzzle6\Client;
use PhpExtended\Endpoint\HttpEndpoint;
use PhpExtended\GeoApiGouvFr\GeoApiGouvFrCommune;
use PhpExtended\GeoApiGouvFr\GeoApiGouvFrEndpoint;
use Psr\Http\Client\ClientExceptionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Autocomplete controller that searches a locality.
 *
 * @package Drupal\address_autocomplete_france\Controller
 */
class LocalityController extends ControllerBase {

  /**
   * HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  private $client;

  /**
   * Time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  private $time;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): LocalityController {
    $instance = parent::create($container);
    $instance->client = $container->get('http_client');
    $instance->time = $container->get('datetime.time');

    return $instance;
  }

  /**
   * Call the geo.api.gouv.fr API.
   *
   * @param string $query
   *   Search query.
   *
   * @return array
   *   API result.
   *
   * @throws \PhpExtended\DataProvider\UnprovidableException
   *
   * @link https://geo.api.gouv.fr/decoupage-administratif/communes
   */
  private function callApi(string $query): array {
    $json = [];

    $endpoint = new GeoApiGouvFrEndpoint(new HttpEndpoint(new Client($this->client)));

    try {
      /** @var \PhpExtended\GeoApiGouvFr\GeoApiGouvFrCommune $result */
      foreach ($endpoint->jsonIteratorGet(
        GeoApiGouvFrCommune::class,
        Url::fromUserInput(
          '/communes', ['query' => ['codePostal' => $query]]
        )->toString()) as $result
      ) {
        $name = $result->getNom();
        $json[] = [
          'label' => $name,
          // This is a bit ugly but the AJAX callback needs both these info.
          'value' => $query . ':' . $name,
        ];
      }
    }
    catch (ClientExceptionInterface $e) {
      $this->getLogger('address_autocomplete_france')
        ->error(t('Could not get locality: @error', ['@error' => $e->getMessage()]));
    }

    return $json;
  }

  /**
   * Search localities by their postal code.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Response.
   *
   * @throws \PhpExtended\DataProvider\UnprovidableException
   */
  public function getByPostalCode(Request $request): JsonResponse {
    $cache = $this->cache();

    $json = [];

    if ($request->query->has('q')) {
      $query = $request->query->get('q');

      // No need to call the API if it is obviously not a postal code.
      if (is_numeric($query) && strlen($query) == 5) {
        $cid = implode(':', [
          'address_autocomplete_france',
          'locality',
          $query,
        ]);

        if ($cached = $cache->get($cid)) {
          $json = $cached->data;
        }
        else {
          $json = $this->callApi($query);

          $cache->set($cid, $json, $this->time->getCurrentTime() + 86400);
        }
      }
    }

    return new JsonResponse($json);
  }

}
